@@ -3,7 +3,7 @@ setClass("aws",
                         y = "array",
                         dy = "numeric",
                         x = "numeric",
-                        ni = "integer",
+                        ni = "numeric",
                         mask = "logical",
                         theta = "array",
                         mae = "numeric",
@@ -26,14 +26,14 @@ setClass("aws",
                         earlystop = "logical",
                         varmodel = "character",
                         vcoef = "numeric",
-                        call = "function")
+                        call = "call")
          )
 setClass("awssegment",
          representation(.Data = "list",
                         y = "array",
                         dy = "numeric",
                         x = "numeric",
-                        ni = "integer",
+                        ni = "numeric",
                         mask = "logical",
                         segment = "array",
                         level = "numeric",
@@ -59,8 +59,84 @@ setClass("awssegment",
                         earlystop = "logical",
                         varmodel = "character",
                         vcoef = "numeric",
-                        call = "function")
+                        call = "call")
          )
+setClass("kernsm",
+         representation(.Data = "list",
+                        y = "array",
+                        dy = "numeric",
+                        x = "numeric",
+                        h = "numeric",
+                        kern = "character",
+                        m = "integer",
+                        nsector = "integer",
+                        sector = "integer",
+                        symmetric = "logical",
+                        yhat = "array",
+                        vred = "numeric",
+                        call = "call")
+         )
+setClass("ICIsmooth",
+         representation(.Data = "list",
+                        y = "array",
+                        dy = "numeric",
+                        x = "numeric",
+                        hmax = "numeric",
+                        hinc = "numeric",
+                        thresh = "numeric",
+                        kern = "character",
+                        m = "integer",
+                        nsector = "integer",
+                        sector = "integer",
+                        symmetric = "logical",
+                        yhat = "array",
+                        vhat = "array",
+                        hbest = "array",
+                        sigma = "numeric",
+                        call = "call")
+         )
+kernsmobj <- function(y,x=numeric(0),h,kern,m,nsector,sector,symmetric,
+                      yhat,vred,call,data=list(NULL)){                                                                
+              dy <- if(is.null(dim(y))) length(y) else dim(y)
+              invisible(new("kernsm",
+                        .Data=data,
+                        y = array(y,dy),
+                        dy = dy,
+                        x = as.numeric(x),
+                        h = as.numeric(h),
+                        kern = as.character(kern),
+                        m = as.integer(m),
+                        nsector = as.integer(nsector),
+                        sector = as.integer(sector),
+                        symmetric = as.logical(symmetric),
+                        yhat = array(yhat,dy),
+                        vred = vred,
+                        call = call)
+            )
+}
+ICIsmoothobj <- function(y,x=numeric(0),h,hinc,thresh,kern,m,nsector,sector,symmetric,
+                      yhat,vhat,hbest,sigma,call,data=list(NULL)){                                                                
+              dy <- if(is.null(dim(y))) length(y) else dim(y)
+              invisible(new("ICIsmooth",
+                        .Data=data,
+                        y = array(y,dy),
+                        dy = dy,
+                        x = as.numeric(x),
+                        hmax = as.numeric(h),
+                        hinc = as.numeric(hinc),
+                        thresh = as.numeric(thresh),
+                        kern = as.character(kern),
+                        m = as.integer(m),
+                        nsector = as.integer(nsector),
+                        sector = if(is.null(sector)) as.integer(0) else as.integer(sector),
+                        symmetric = as.logical(symmetric),
+                        yhat = array(yhat,dy),
+                        vhat = array(vhat,dy),
+                        hbest = array(hbest,dy),
+                        sigma = as.numeric(sigma),
+                        call = call)
+            )
+}
 
 awsobj <- function(y,theta,var,hmax,sigma2,lkern,lambda,ladjust,aws,memory,
 call,homogen=FALSE,earlystop=FALSE,family="Gaussian",degree=0,
@@ -73,7 +149,7 @@ varmodel="Constant",vcoef=NULL,mask=logical(0),data=list(NULL)){
                         y = array(y,dy),
                         dy = dy,
                         x = as.numeric(x),
-                        ni = as.integer(ni),
+                        ni = as.numeric(ni),
                         mask = as.logical(mask),
                         theta = array(theta,c(dy,degree+1)),
                         mae = as.numeric(mae),
@@ -99,7 +175,8 @@ varmodel="Constant",vcoef=NULL,mask=logical(0),data=list(NULL)){
                         call = call)
             )
 }
-awssegmentobj <- function(y,theta,segment,var,level,delta,hmax,sigma2,lkern,lambda,ladjust,aws,memory,
+awssegmentobj <- function(y,theta,segment,var,level,delta,hmax,sigma2,
+          lkern,lambda,ladjust,aws,memory,
 call,homogen=FALSE,earlystop=FALSE,family="Gaussian",degree=0,
 x=numeric(0),ni=as.integer(1),xmin=numeric(0),xmax=numeric(0),
 wghts=numeric(0),scorr=0,mae=numeric(0),shape=numeric(0),
@@ -110,7 +187,7 @@ varmodel="Constant",vcoef=NULL,mask=logical(0),data=list(NULL)){
                         y = array(y,dy),
                         dy = dy,
                         x = as.numeric(x),
-                        ni = as.integer(ni),
+                        ni = as.numeric(ni),
                         mask = as.logical(mask),
                         segment = array(as.integer(segment),dy),
                         level = as.numeric(level),
