@@ -1,33 +1,3 @@
-######################################################################
-#
-#  set numer of cores (code from package spMC)
-#
-#####################################################################
-setCores1 <-
-function(n) {
-  # Set number of CPU cores which will be used by the package
-  #
-  #     n number of CPU cores
-
-  if (!missing(n)) {
-    if (is.numeric(n)) {
-      n <- as.integer(ceiling(n))
-      n <- .C('setNumSlaves', n = as.integer(n), DUP = FALSE, PACKAGE = "aws")$n
-    }
-  }
-  n <- 0L
-  crTot <- 0L
-  n <- .C('getNumSlaves', n = as.integer(n), DUP = FALSE, PACKAGE = "aws")$n
-  if (n <= 1L) {
-    cat("Parallel computation will not perform. CPU cores in use: 1.\n")
-  }
-  else {
-    crTot <- .C('getNumCores', n = as.integer(crTot), DUP = FALSE, PACKAGE = "aws")$n
-    cat("Parallel computation will perform.\n")
-    cat("  Total CPU cores available: ", crTot, ".\n", sep = "")
-    cat("  CPU cores in use: ", n, ".\n", sep = "")
-  }
-}
 #########################################################################
 #
 #   binning in 1D -- 3D (adapted from binning function in package sm
